@@ -20,8 +20,8 @@
 #include <bakery/App/Dialog_OfferSave.h>
 #include <bakery/App/Dialog_OverwriteFile.h>
 #include <gconfmm.h>
+#include <libgnomevfsmm.h>
 #include <libintl.h>
-#include <fstream>
 #include <algorithm>
 
 //#include <gtk/gtkfilesel.h>
@@ -71,7 +71,7 @@ void App_WithDoc::on_menu_file_close()
 }
 
 
-bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
+bool App_WithDoc::open_document(const Glib::ustring& file_uri)
 {
   //Check whether it's already open:
   //It could even be open in this instance.
@@ -85,7 +85,7 @@ bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
     if(pApp)
     {
       Document* pDoc = pApp->get_document();
-      if(pDoc->get_filepath() == strFilepath)
+      if(pDoc->get_file_uri() == file_uri)
       {
         bAlreadyOpen = true;
         pAppAlreadyOpen = pApp;
@@ -126,7 +126,7 @@ bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
     }
 
     //Open it.
-    pApp->m_pDocument->set_filepath(strFilepath);
+    pApp->m_pDocument->set_file_uri(file_uri);
     bool bTest = pApp->m_pDocument->load();
 
     if(!bTest) //if open failed.
@@ -157,7 +157,7 @@ bool App_WithDoc::open_document(const Glib::ustring& strFilepath)
       set_document_modified(false); //disables menu and toolbar Save items.
 
       //Update document history list:
-      document_history_add(strFilepath);
+      document_history_add(file_uri);
 
       return true; //success.
     }
@@ -174,9 +174,9 @@ void App_WithDoc::on_menu_file_open()
   ui_bring_to_front();
 
   //Ask user to choose file to open:
-  Glib::ustring strFilepath = ui_file_select_open();
-  if(!strFilepath.empty())
-    open_document(strFilepath);
+  Glib::ustring file_uri = ui_file_select_open();
+  if(!file_uri.empty())
+    open_document(file_uri);
 }
 
 //This is for calling directly, use the other override for the signal handler:
@@ -194,29 +194,36 @@ void App_WithDoc::on_menu_file_saveas()
   ui_bring_to_front();
 
   //Show the save dialog:  
-  const Glib::ustring& strFilepathOld = m_pDocument->get_filepath();
-  Glib::ustring strFilepath = ui_file_select_save(strFilepathOld);
-  if(!strFilepath.empty())
+  const Glib::ustring& file_uriOld = m_pDocument->get_file_uri();
+  Glib::ustring file_uri = ui_file_select_save(file_uriOld);
+  if(!file_uri.empty())
   {
-    bool bUseThisFilePath = true;
+    bool bUseThisFileUri = true;
 
     //Check whether file exists already:
     {
-      std::ifstream fStream(strFilepath.c_str());
-      if(fStream.is_open())
+      // Try to open the input file.
+      Gnome::Vfs::Handle read_handle;
+      try
       {
-        //Ask user to confirm overwrite:
-        bool bOverwrite = ui_ask_overwrite(strFilepath);
+        read_handle.open(file_uri, Gnome::Vfs::OPEN_READ);
+
+        //It does (there was no exception), so ask the user to confirm overwrite:
+        bool bOverwrite = ui_ask_overwrite(file_uri);
 
         //Respond to button that was clicked:
-        bUseThisFilePath = bOverwrite;
+        bUseThisFileUri = bOverwrite;
+      }
+      catch(const Gnome::Vfs::exception& ex)
+      {
+        bUseThisFileUri = true; //It does not exist.
       }
     }
 
     //Save to this filepath:
-    if(bUseThisFilePath)
+    if(bUseThisFileUri)
     {
-      m_pDocument->set_filepath(strFilepath, true); //true = enforce file path
+      m_pDocument->set_file_uri(file_uri, true); //true = enforce file path
       bool bTest  = m_pDocument->save();
 
       if(!bTest)
@@ -256,7 +263,7 @@ void App_WithDoc::on_menu_file_save()
   if(m_pDocument)
   {
     //If there is already a filepath, then save to that location:
-    if(!(m_pDocument->get_filepath().empty()))
+    if(!(m_pDocument->get_file_uri().empty()))
     {
       bool bTest = m_pDocument->save();
 
@@ -454,7 +461,7 @@ void App_WithDoc::after_successful_save()
   set_document_modified(false); //enables/disables menu and toolbar widgets.
     
   //Update document history list:
-  document_history_add(m_pDocument->get_filepath());     
+  document_history_add(m_pDocument->get_file_uri());     
 }
 
 Glib::ustring App_WithDoc::get_conf_fullkey(const Glib::ustring& key)
@@ -463,12 +470,12 @@ Glib::ustring App_WithDoc::get_conf_fullkey(const Glib::ustring& key)
 }
 
 
-void App_WithDoc::document_history_add(const Glib::ustring& strFilepath)
+void App_WithDoc::document_history_add(const Glib::ustring& file_uri)
 {
   //Override this.
 }
 
-void App_WithDoc::document_history_remove(const Glib::ustring& strFilepath)
+void App_WithDoc::document_history_remove(const Glib::ustring& file_uri)
 {
   //Override this.
 }
