@@ -17,12 +17,15 @@
  */
 
 #include "bakery/Document/Document.h"
-#include <fstream>
+#include <libgnomevfsmm.h>
+//#include <fstream>
 #include <libintl.h>
 
 namespace Bakery
 {
 
+const guint BYTES_TO_PROCESS = 256;
+
 Document::Document()
 {
   m_bIsNew = true;
@@ -37,17 +40,17 @@ Document::~Document()
   signal_forget_.emit();
 }
 
-Glib::ustring Document::get_filepath() const
+Glib::ustring Document::get_file_uri() const
 {
-  return m_strFilePath;
+  return m_file_uri;
 }
 
-void Document::set_filepath(const Glib::ustring& strVal, bool bEnforceFileExtension /* = false */)
+void Document::set_file_uri(const Glib::ustring& file_uri, bool bEnforceFileExtension /* = false */)
 {
-  if(strVal != m_strFilePath)
+  if(file_uri != m_file_uri)
     set_modified(); //Ready to save() for a Save As.
 
-  m_strFilePath = strVal;
+  m_file_uri = file_uri;
 
   //Enfore file extension:
   if(bEnforceFileExtension)
@@ -57,20 +60,20 @@ void Document::set_filepath(const Glib::ustring& strVal, bool bEnforceFileExtens
       bool bAddExt = false;
       Glib::ustring strExt = "." + get_file_extension();
       
-      if(m_strFilePath.size() < strExt.size()) //It can't have the ext already if it's not long enough.
+      if(m_file_uri.size() < strExt.size()) //It can't have the ext already if it's not long enough.
       {
         bAddExt = true; //It isn't there already.
       }
       else
       {
-        Glib::ustring strEnd = m_strFilePath.substr(m_strFilePath.size()-strExt.size());
+        Glib::ustring strEnd = m_file_uri.substr(m_file_uri.size()-strExt.size());
         if(strEnd != strExt) //If it doesn't already have the extension
           bAddExt = true;
       }
  
       //Add extension if necessay.
       if(bAddExt)
-        m_strFilePath += strExt;
+        m_file_uri += strExt;
         
       //Note that this does not replace existing extensions, so it could be e.g. 'something.blah.theext'
     }
@@ -157,26 +160,51 @@ bool Document::read_from_disk()
 {
   m_strContents.erase();
 
-  //Read the text from the disk:
-  //TODO: Check for read only.
-  std::ifstream fStream(m_strFilePath.c_str());
-  if(fStream.is_open())
+  // open the input file for read access
+  Gnome::Vfs::Handle read_handle;
+
+  try
   {
-    while(!(fStream.eof()))
+    read_handle.open(m_file_uri, Gnome::Vfs::OPEN_READ);
+  }
+  catch(const Gnome::Vfs::exception& ex)
+  {
+    // If the operation was not successful, print the error and abort
+    return false; //print_error(ex, input_uri_string);
+  }
+
+  // Read data from the input uri:
+  guint buffer[BYTES_TO_PROCESS] = {0, }; // For each chunk.
+  Gnome::Vfs::FileSize bytes_read = 0;
+  std::string data; //We use a std::string because we might not get whole UTF8 characters at a time. This might not be necessary.
+
+  try
+  {
+    bool bContinue = true;
+    while(bContinue)
     {
-      char chTemp = fStream.get();
-      if(!(fStream.eof()))
-        m_strContents += chTemp;
+      bytes_read = read_handle.read(buffer, BYTES_TO_PROCESS);
+
+      if(bytes_read == 0)
+        bContinue = false; //stop because we reached the end.
+      else
+      {
+        // Add the text to the string:
+        data += std::string((char*)buffer, bytes_read);
+      }
     }
-
-    set_modified(false);
-
-    return true; //Success.
   }
-  else
+  catch(const Gnome::Vfs::exception& ex)
   {
-    return false; //Failed.
+    // If the operation was not successful, print the error and abort
+    return false; //print_error(ex, input_uri_string);
   }
+
+  m_strContents = data;
+
+  set_modified(false);
+
+  return true; //Success.
 }
 
 bool Document::write_to_disk()
@@ -184,55 +212,66 @@ bool Document::write_to_disk()
   //Write the changed data to disk:
   if(get_modified())
   {
-    std::ofstream file(m_strFilePath.c_str());
-    if(file)
-    {
-      //TODO: UTF8
-      file << (std::string)m_strContents;
-      
-      set_modified(false);
-      set_read_only(false); //It's obviously not read-only if we can write to it.
+    /* we use create instead of open, because open will not create the file if it does
+    not already exist. The last argument is the permissions to use if the file is created,
+    the second to last tells GnomeVFS that its ok if the file already exists, and just open it */
 
-      return true; //Success.
-    }
-    else
+    Gnome::Vfs::Handle write_handle;
+      
+    try
     {
-      return false; //Failed.
+      write_handle.create(m_file_uri, Gnome::Vfs::OPEN_WRITE, false, 0x777);
     }
-  }
-  else
-  {
+    catch(const Gnome::Vfs::exception& ex)
+    {
+     // If the operation was not successful, print the error and abort
+     return false; // print_error(ex, output_uri_string);
+    }
+
+    try
+    {
+      //Write the data to the output uri
+      GnomeVFSFileSize bytes_written = write_handle.write(m_strContents.data(), m_strContents.bytes());
+    }
+    catch(const Gnome::Vfs::exception& ex)
+    {
+      // If the operation was not successful, print the error and abort
+      return false; //print_error(ex, output_uri_string);
+    }
+  
     return true; //Success. (At doing nothing, because nothing needed to be done.)
   }
+  else
+    return true; //Success. (At doing nothing, because nothing needed to be done.)
 }
 
 Glib::ustring Document::get_name() const
 {
-  return util_filepath_get_name(m_strFilePath);
+  return util_file_uri_get_name(m_file_uri);
 }
 
-Glib::ustring Document::util_filepath_get_name(const Glib::ustring& filePath)
+Glib::ustring Document::util_file_uri_get_name(const Glib::ustring& file_uri)
 {
   Glib::ustring strResult;
 
-  if(filePath.size())
+  if(file_uri.size())
   {
     //Find position of last '/' character.
-    Glib::ustring::size_type posLastSlash = filePath.find_last_of("/");
+    Glib::ustring::size_type posLastSlash = file_uri.find_last_of("/");
     if(posLastSlash == Glib::ustring::npos)
     {
       //No '/' found. The file must be in the current or top-level directory.
-      strResult = filePath;
+      strResult = file_uri;
     }
     else
     {
       //'/' found. The filename is the text after this: e.g. sub/sub/filename:
-      strResult = filePath.substr(posLastSlash+1);
+      strResult = file_uri.substr(posLastSlash+1);
     }
   }
 
   //Show untitled explicitly:
-  //Also happens for filepaths with path but no name. e.g. /sub/sub/, which shouldn't happen.
+  //Also happens for file_uris with path but no name. e.g. /sub/sub/, which shouldn't happen.
   if(strResult.size() == 0)
     strResult = gettext("Untitled");
 
@@ -251,6 +290,30 @@ ViewBase* Document::get_view()
 
 bool Document::get_read_only() const
 {
+  if(m_bReadOnly)
+  {
+    //An application might have use set_read_only() to make this document explicitly read_only, regardless of the positions of the storage location.
+    return true;
+  }
+  else
+  {
+    if(m_file_uri.empty())
+      return false; //It must be a default empty document, not yet saved, so it is not read-only.
+    else
+    {
+      try
+      {
+        Gnome::Vfs::FilePermissions permissions = Gnome::Vfs::Handle::get_file_info(m_file_uri, Gnome::Vfs::FILE_INFO_GET_ACCESS_RIGHTS)->get_permissions();
+        bool read_only = ((permissions & Gnome::Vfs::PERM_ACCESS_WRITABLE) != Gnome::Vfs::PERM_ACCESS_WRITABLE);
+        return read_only;
+      }
+      catch(const Gnome::Vfs::exception& ex)
+      {
+        return false; //We should at least be able to read the permissions, so maybe the location is invalid. I'm not sure what the best return result here is.
+      }
+    }
+  }
+  
   return m_bReadOnly;
 }
 
